<?php
namespace MatthiasWeb\RealThumbnailGenerator\editor;
use MatthiasWeb\RealThumbnailGenerator\attachment;
use MatthiasWeb\RealThumbnailGenerator\editor;

defined('ABSPATH') or die('No script kiddies please!'); // Avoid direct file request

/**
 * Use this trait in your extended class for the below methods.
 * You have to redefine the resize / _resize method yourself or use GDExtend / ImagickExtend.
 */
trait Extend {
    private $adapter;

    public function extendWithAdapter() {
        $this->adapter = new editor\Adapter($this, $this->file);
    }

    public function generate_filename($suffix = null, $dest_path = null, $extension = null) {
        $result = $this->adapter->generate_filename($this->size, $dest_path, $extension);
        if ($result === false) {
            // There was an error, use default WP naming
            $result = parent::generate_filename($suffix, $dest_path, $extension);
        }
        return $result;
    }

    public function multi_resize($sizes) {
        $metadata = parent::multi_resize($sizes);
        return $this->adapter->multi_resize($metadata, $sizes);
    }

    protected function _save($image, $filename = null, $mime_type = null) {
        $resized = $this->adapter->checkSave();
        if ($resized === false) {
            $resized = parent::_save($image, $filename, $mime_type); // Call the original function to save the image to disk
        } else {
            $this->generate_filename(); // We must generate the filename to fill the data
        }
        return $this->adapter->_save($resized, $image, $filename, $mime_type);
    }
}
