<?php
namespace MatthiasWeb\RealThumbnailGenerator\general;
use MatthiasWeb\RealThumbnailGenerator\base;
use MatthiasWeb\RealThumbnailGenerator\rest;
use MatthiasWeb\RealThumbnailGenerator\attachment;
use MatthiasWeb\RealThumbnailGenerator\editor;
use MatthiasWeb\RealThumbnailGenerator\overrides\interfce;
use MatthiasWeb\RealThumbnailGenerator\lite;

defined('ABSPATH') or die('No script kiddies please!'); // Avoid direct file request

// Include files, where autoloading is not possible, yet
require_once RTG_INC . 'base/Core.class.php';
require_once RTG_INC . 'overrides/interfce/IOverrideCore.interface.php';
require_once RTG_OVERRIDES_INC . 'Core.trait.php';

/**
 * Singleton core class which handles the main system for plugin. It includes
 * registering of the autoloader, all hooks (actions & filters) (see base\Core class).
 */
class Core extends base\Core implements interfce\IOverrideCore {
    use lite\Core;

    /**
     * Singleton instance.
     */
    private static $me;

    /**
     * The main service.
     *
     * @see rest\Service
     */
    private $service;

    /**
     * Application core constructor.
     */
    protected function __construct() {
        parent::__construct();

        // Register all your before init hooks here
        add_action('plugins_loaded', [$this, 'updateDbCheck']);
        add_filter('wp_die_ajax_handler', [attachment\Regenerate::getInstance(), 'wp_die']);
        add_filter('wp_die_handler', [attachment\Regenerate::getInstance(), 'wp_die']);
        add_filter('rest_post_dispatch', [attachment\Regenerate::getInstance(), 'wp_die'], 10, 1);

        $this->overrideConstruct();
    }

    /**
     * The init function is fired even the init hook of WordPress. If possible
     * it should register all hooks to have them in one place.
     */
    public function init() {
        $this->service = new rest\Service();

        //add_image_size('favicon', 32, 32, true);
        //add_image_size('another', 64, 64, true);

        // Register all your hooks here
        add_action('rest_api_init', [$this->getService(), 'rest_api_init']);
        add_action('rest_api_init', [new rest\Analyse(), 'rest_api_init']);
        add_action('admin_enqueue_scripts', [$this->getAssets(), 'admin_enqueue_scripts']);
        add_action('wp_enqueue_scripts', [$this->getAssets(), 'wp_enqueue_scripts']);
        add_action('delete_attachment', [attachment\Thumbnail::getInstance(), 'delete_attachment']);

        add_filter('override_load_textdomain', [new Localization(), 'override_load_textdomain'], 10, 3);
        add_filter('load_script_translation_file', [new Localization(), 'load_script_translation_file'], 10, 3);
        add_filter('media_row_actions', [View::getInstance(), 'media_row_actions'], 10, 2);
        add_filter('attachment_fields_to_edit', [View::getInstance(), 'attachment_fields_to_edit'], 10, 2);
        add_filter(
            'wp_update_attachment_metadata',
            [attachment\Regenerate::getInstance(), 'wp_update_attachment_metadata'],
            10,
            2
        );
        add_filter('wp_image_editors', [editor\Editor::getInstance(), 'wp_image_editors'], PHP_INT_MAX);
        add_filter(
            'wp_generate_attachment_metadata',
            RTG_NS . '\\editor\\Adapter::wp_generate_attachment_metadata',
            PHP_INT_MAX,
            2
        );
        add_filter('crop_thumbnails_filename', RTG_NS . '\\editor\\Adapter::crop_thumbnails_filename', 11, 5);
        add_filter(
            'crop_thumbnails_before_update_metadata',
            RTG_NS . '\\editor\\Adapter::crop_thumbnails_before_update_metadata',
            11
        );

        $this->overrideInit();
    }

    /**
     * Get the service.
     *
     * @return rest\Service
     */
    public function getService() {
        return $this->service;
    }

    /**
     * Get singleton core class.
     *
     * @return Core
     */
    public static function getInstance() {
        return !isset(self::$me) ? (self::$me = new Core()) : self::$me;
    }
}
